@@ -2,36 +2,30 @@
 """
    Python client to connect to the CMS via a local or remote gRPC socket.
 """
+from __future__ import print_function, absolute_import, unicode_literals
+
+import json
 import os
 import ssl
-import sys
-import json
-import time
-import grpc
-import itertools
-
+from distutils.version             import LooseVersion
 from threading                     import Timer
-from distutils.version             import StrictVersion, LooseVersion
-
-from dgpy.safehaven.api            import api_pb2
-from dgpy.safehaven.api            import api_pb2_grpc
 
+import grpc
 from dgpy                          import version
-from dgpy.user                     import User
-from dgpy.errors                   import *
-from dgpy.job                      import job_is_done
-from dgpy.dvol                     import Dvol, DvolCreate, RecoverySequence
-from dgpy.interceptor              import access_token_auth
-
-from dgpy.aggregates.disk          import disk_pb2
-from dgpy.aggregates.dvol          import dvol_pb2, dvol_enums_pb2
-from dgpy.aggregates.site          import site_pb2
 from dgpy.aggregates.cluster       import cluster_pb2
-from dgpy.safehaven.cron           import cron_pb2
-from dgpy.safehaven.api.types      import types_enums_pb2
-from dgpy.safehaven.jobsystem      import jobsystem_pb2
+from dgpy.aggregates.disk          import disk_pb2
+from dgpy.aggregates.dvol          import dvol_pb2
+from dgpy.aggregates.site          import site_pb2
+from dgpy.dvol                     import Dvol, RecoverySequence
+from dgpy.errors                   import handle_exceptions, LegacyMessageError, GrpcError, TimeOutError, NotFoundError
+from dgpy.interceptor              import access_token_auth
+from dgpy.job                      import job_is_done
 from dgpy.messaging.dgcomm.message import syntropy_message_pb2
 from dgpy.messaging.dgcomm.message.protocol_number import ProtoNum
+from dgpy.safehaven.api import api_pb2, api_pb2_grpc
+from dgpy.safehaven.cron           import cron_pb2
+from dgpy.safehaven.jobsystem      import jobsystem_pb2
+from dgpy.user                     import User
 
 
 class CmsClient(object):
@@ -82,13 +76,19 @@ class CmsClient(object):
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
-        if self._channel is not None:
-            self._channel.close()
+        self._teardown()
         return False
 
     def __del__(self):
-        if self._channel is not None:
-            self._channel.close()
+        self._teardown()
+
+    def _teardown(self):
+        """Close internal channel if it still exists."""
+        try:
+            if self._channel is not None:
+                self._channel.close()
+        except (TypeError, AttributeError):  # Channel might be gone already
+            pass
 
     #
     # Connection Handling
@@ -197,7 +197,7 @@ class CmsClient(object):
         """
         return self.stub.UiCmsLegacyCommand(syntropy_message_pb2.Message(
             proto_num = proto_num.value,
-            params    = map(str, parameters),
+            params    = list(map(str, parameters)),
         ))
 
     @handle_exceptions
@@ -210,7 +210,7 @@ class CmsClient(object):
         """
         res = self.stub.UiCmsLegacyMessage(syntropy_message_pb2.Message(
             proto_num = proto_num.value,
-            params    = map(str, parameters),
+            params    = list(map(str, parameters)),
         ))
         if ProtoNum(res.proto_num).IsErr():
             # The exception cause is in most cases the last member of the res.params array.
@@ -556,7 +556,7 @@ class CmsClient(object):
         """
         res = self.legacy_query(ProtoNum.MSG_UI_CMS_LIST_DVOL_SNAP, dvolName, ('MasterSite', 'RemoteSite')[remote])
         # Caveat: response may be empty or represented as JSON 'null' - always return proper list type.
-        return (res and json.loads(res[0])['snapshots']) or []
+        return json.loads(res[0])['snapshots'] if res else []
 
     def dvol_list_targets(self, dvolName, remote=False):
         """
@@ -571,7 +571,7 @@ class CmsClient(object):
         # Parameter order in response message as per dvol_sbd_list_targets_to_msg:
         columns = ('iqn', 'vm_name', 'os_category', 'os_version', 'mirror_type')
         res = self.legacy_query(ProtoNum.MSG_UI_CMS_DVOL_LIST_TARGETS, dvolName, ('MasterSite', 'RemoteSite')[remote])
-        return [Dvol.Target(itertools.izip(columns, l.split('\t'))) for l in res]
+        return [Dvol.Target(list(zip(columns, l.split('\t')))) for l in res]
 
     def dvol_target_delete(self, dvolName, targetIqn):
         """Remove the target identified by @targetIqn from @dvolName."""
@@ -598,7 +598,7 @@ class CmsClient(object):
         """Return the recovery plan (sequence of recovery steps) of @dvolName on @srnIp."""
         res = self.legacy_query(ProtoNum.MSG_UI_CMS_DVOL_LIST_RECOVERY_PLAN, dvolName, srnIp)
          # FIXME: crazy format - need to special-case the tab
-        return RecoverySequence.from_string("\n".join(map(lambda s: s.replace('\t', ',', 1), res)))
+        return RecoverySequence.from_string("\n".join([s.replace('\t', ',', 1) for s in res]))
 
     def dvol_create_or_update_recovery_plan(self, dvolName, srnIp, rp):
         """Set @rp as recovery plan for @dvolName on SRN @srnIp.
@@ -644,21 +644,21 @@ class CmsClient(object):
             retention_policy = rp,
         ))
 
-    def dvol_manual_snapshot(self, dvolName, vssTgts=[], stopSync=False):
+    def dvol_manual_snapshot(self, dvolName, vssTgts=None, stopSync=False):
         """Perform a manual checkpoint on the Legacy Protection Group @dvolName:
            @vssTargets: optional list of VSS-enabled targets,
            @stopSync:   whether to (temporarily) disable synchronisation and periodic snaphots (deprecated).
         Note that @vssTargets only makes sense on WINDOWS PGs, and is currently restricted to 1 target.
         """
-        return self.legacy_command(ProtoNum.MSG_UI_CMS_DVOL_MANUAL_SNAPSHOT, dvolName, ('false','true')[stopSync], ",".join(vssTgts))
+        return self.legacy_command(ProtoNum.MSG_UI_CMS_DVOL_MANUAL_SNAPSHOT, dvolName, ('false','true')[stopSync], ",".join(vssTgts or []))
 
     @handle_exceptions
-    def pg_manual_checkpoint(self, dvolName, vssTgts=[]):
+    def pg_manual_checkpoint(self, dvolName, vssTgts=None):
         """Perform a manual checkpoint on the Non-Legacy Protection Group @dvolName (analogous to dvol_manual_snapshot):
            @vssTargets: optional list of VSS-enabled targets (Windows only)
         """
         # FIXME: vssTargets should be a list, see https://github.com/CenturyLinkCloud/sh-safehaven3/issues/3972
-        return self.stub.PgManualCheckpoint(dvol_pb2.PgManualCheckpointRequest(name=dvolName, vss_targets=",".join(vssTgts)))
+        return self.stub.PgManualCheckpoint(dvol_pb2.PgManualCheckpointRequest(name=dvolName, vss_targets=",".join(vssTgts or [])))
 
     def dvol_test_failover(self, dvolName, snap_id, temp_size, power_on=False, isolate_network=False):
         """Create a temporary test-failover recovery image from a checkpoint
@@ -672,7 +672,7 @@ class CmsClient(object):
         isolate_net = ('no', 'yes')[isolate_network]       # defines.h: NO_STR / YES_STR
         return self.legacy_command(ProtoNum.MSG_UI_CMS_DVOL_CREATE_BACKUP_IMAGE, dvolName, snap_id, temp_size, pwr_action, isolate_net)
 
-    def dvol_test_failover_delete(self, dvolName, temp_size, power_off=False):
+    def dvol_test_failover_delete(self, dvolName, _temp_size, power_off=False):
         """Tear down the test-failover image on @dvolName.
            @dvolName:  name of the Dvol that has the test-failover image
            @power_off: whether to power off the remote site before deleting the image
@@ -696,12 +696,12 @@ class CmsClient(object):
     @handle_exceptions
     def pg_migrate(self, pg_migrate_request):
         """Migrate a non-legacy PG using @pg_migrate_request (a dvol_pb2.PgMigrateRequest)."""
-        return self.stub.PgMigrate(pg_migrate_request);
+        return self.stub.PgMigrate(pg_migrate_request)
 
     @handle_exceptions
     def pg_resize(self, pg_resize_request):
         """Resize a non-legacy PG using @pg_resize_request (a dvol_pb2.PgResizeRequest)."""
-        return self.stub.PgResize(pg_resize_request);
+        return self.stub.PgResize(pg_resize_request)
 
     #
     # Ansible Commands:
@@ -732,4 +732,4 @@ class CmsClient(object):
         return self.legacy_command(ProtoNum.MSG_UI_CMS_DVOL_INSTALL_MAKESTUB, dvolName, vms, ips, users, passwds, url, stub_os)
 
 if __name__ == "__main__":
-    print "Python SDK Version:", CmsClient.version()
+    print("Python SDK Version:", CmsClient.version())
