@@ -1,18 +1,23 @@
 #
 # Abstractions and routines dealing with Distributed Volumes (Dvols).
 #
-import re
+from __future__ import absolute_import, unicode_literals
+
 import os
+import re
+import string
 import time
 import types
-import string
-import logging
-import collections
+import functools
+from collections import namedtuple, Iterable, OrderedDict
+try:
+    from UserList import UserList
+except ImportError:
+    from collections import UserList
 
 import dgpy.utils
-from UserList    import UserList
-from conf_file   import ConfigDict, SyntropyValue
-from collections import namedtuple, Iterable
+from dgpy.conf_file   import ConfigDict, syntropy_value
+
 
 #
 # Configuration file syntax-validation functions.
@@ -24,7 +29,7 @@ def _bool(v):
     return _string(v) in ('true', 'false')
 
 def _number(v):
-    return re.match('\s*\d+\s*$', _string(v))
+    return re.match(r'\s*\d+\s*$', _string(v))
 
 def _ipaddress(v):
     return dgpy.utils.is_valid_ip(_string(v))
@@ -85,7 +90,7 @@ class Dvol(ConfigDict):
     # Dvol leg role
     LEG_ROLE = ( "MASTER_LEG", "REMOTE_LEG" )
 
-    ATTR_MAP = collections.OrderedDict([
+    ATTR_MAP = OrderedDict([
         ('dvol_name',                           _string),
         ('type',                                lambda v: v in Dvol.TYPE),
         ('role',                                lambda v: v in Dvol.LEG_ROLE),
@@ -206,7 +211,7 @@ class Dvol(ConfigDict):
         # as in defines.h
         IQN_PREFIX = 'iqn.2016-09.io.ctl'
 
-        ATTR_MAP = collections.OrderedDict([
+        ATTR_MAP = OrderedDict([
             ('dvol_name',                   _string),
             ('vm_name',                     _string),
             ('iqn',                         lambda v: _string(v).startswith(Dvol.Target.IQN_PREFIX)),
@@ -241,6 +246,9 @@ class Dvol(ConfigDict):
         def __eq__(self, other):
             return ConfigDict.__eq__(self, other) and self.pluns == other.pluns
 
+        def __hash__(self):
+            return ConfigDict.__hash__(self) + hash(str(self.pluns))
+
         def add_plun(self, config_dict):
             """Add target PLUN generated from @config_dict."""
             plun = Dvol.Target.PLUN(config_dict)
@@ -260,7 +268,7 @@ class Dvol(ConfigDict):
             # from lib_utils.c
             SCSI_SERIAL_NUMBER_PREFIX = 'SDGDvolD'
 
-            ATTR_MAP = collections.OrderedDict([
+            ATTR_MAP = OrderedDict([
                 ('plun_name',      _string),
                 ('plun_lunid',     _number),
                 ('size',           _lvmsize),
@@ -274,7 +282,7 @@ class Dvol(ConfigDict):
             # Allow SCSISNs to differ only in their disk_id.
             # (Since the disk_ids in MS/RS pairs are not identical in Syntropy, but consecutive).
             def __hash__(self):
-                return hash(repr([(k, v[:-2] if k == 'scsisn' else v) for k,v in self.iteritems()]))
+                return hash(repr([(k, v[:-2] if k == 'scsisn' else v) for k,v in self.items()]))
 
 
     #--------------------------------------------------------------------------------------------------------------
@@ -306,12 +314,12 @@ class Dvol(ConfigDict):
                       '%s_sbd_device', 'sbd_%s_disk', 'sbd_%s_snapshots_disk']
         site_keys  = ['%s_site_type', '%s_stubs', '%s_vnames']
 
-        master = Dvol.SRN(*map(config_dict.get,
-                               map(lambda s: s.replace('%s', 'ms'), conf_keys) + \
-                               map(lambda s: s.replace('%s', 'primary' if ms_is_prim else 'secondary'), site_keys)))
-        remote = Dvol.SRN(*map(config_dict.get,
-                               map(lambda s: s.replace('%s', 'rs'), conf_keys) + \
-                               map(lambda s: s.replace('%s', 'secondary' if ms_is_prim else 'primary'), site_keys)))
+        master = Dvol.SRN(*list(map(config_dict.get,
+                               [s.replace('%s', 'ms') for s in conf_keys] + \
+                               [s.replace('%s', 'primary' if ms_is_prim else 'secondary') for s in site_keys])))
+        remote = Dvol.SRN(*list(map(config_dict.get,
+                               [s.replace('%s', 'rs') for s in conf_keys] + \
+                               [s.replace('%s', 'secondary' if ms_is_prim else 'primary') for s in site_keys])))
 
         if ms_is_prim and not remote.disk:
             raise Exception("Remote site %s does not have a disk" % remote.service_ip)
@@ -386,6 +394,9 @@ class Dvol(ConfigDict):
             return False
         return self.local == other.remote and self.remote == other.local and self.targets == other.targets
 
+    def __hash__(self):
+        return ConfigDict.__hash__(self) + hash(str(self.targets))
+
     def __str__(self):
         return "%s(name=%r, type=%s, role=%r, state=%r, targets=[%s], local=%s, remote=%s)" % (self.__class__.__name__,
                 self.name, self['type'],
@@ -509,7 +520,7 @@ class DvolProperties(namedtuple('DvolProperties', [
         """Enforce the same output as generated by the console.
         Used for compliance testing.
         """
-        return re.sub('^\s*', '', '''
+        return re.sub(r'^\s*', '', '''
           Name                              : %s
           Status                            : %s
           Role                              : %s
@@ -565,7 +576,8 @@ class DvolProperties(namedtuple('DvolProperties', [
         return self.status == 'RUNNING' and self.active_connection == '1'
 
 
-class ArgumentError(SyntaxError): pass
+class ArgumentError(SyntaxError):
+    pass
 
 class DvolCreate(namedtuple('DvolCreate', [
         'name',             # Human-readable name for the Dvol (dvol_name)
@@ -637,7 +649,7 @@ class DvolCreate(namedtuple('DvolCreate', [
 
         # 2. Argument type validation
         if validated_args['type'].upper() not in Dvol.TYPE:
-             raise ArgumentError("Invalid Dvol type %r" % validated_args['type'])
+            raise ArgumentError("Invalid Dvol type %r" % validated_args['type'])
 
         for pg in ('pg_prim_type', 'pg_sec_type'):
             if validated_args[pg] not in Dvol.PG_SITE_TYPE:
@@ -677,11 +689,11 @@ class DvolCreate(namedtuple('DvolCreate', [
         # Prepopulate the stub-provisioning part with empty defaults
         # FIXME: this is a work-around and should be removed soon.
         def_val = lambda key: 'false' if key == 'make_stubs' else ''
-        kwargs  = { k:(SyntropyValue(config[k]) if k in config else def_val(k)) for k in cls._fields }
+        kwargs  = { k:(syntropy_value(config[k]) if k in config else def_val(k)) for k in cls._fields }
         return cls(**kwargs)
 
     def __repr__(self):
-        return re.sub('^\s*', '', '''
+        return re.sub(r'^\s*', '', '''
           Dvol Name                    : %s
           Type                         : %s
           Size                         : %s
@@ -821,8 +833,8 @@ class RecoveryPlan(namedtuple('RecoveryPlan', [
         if not dgpy.utils.is_valid_ip(res_ip):
             raise ArgumentError("%r is not a valid IP" % res_ip)
         return super(RecoveryPlan, cls).__new__(cls, res_name,
-                                                int(up_index), up_action, long(up_delay),
-                                                int(down_index), down_action, long(down_delay),
+                                                int(up_index), up_action, int(up_delay),
+                                                int(down_index), down_action, int(down_delay),
                                                 res_desc, res_ip, group_id, group_name, target_name)
     def __lt__(self, other):
         return self.up_index < other.up_index
@@ -871,10 +883,10 @@ class RecoverySequence(UserList):
     def __init__(self, *args):
         def flatten(acc, l):
             """Helper function to deal with mixed arguments."""
-            acc.extend(l) if isinstance(l, Iterable) else acc.append(l)
+            _ = acc.extend(l) if isinstance(l, Iterable) else acc.append(l)
             return acc
         # validate arguments
-        seq = sorted(reduce(flatten, args, []))
+        seq = sorted(functools.reduce(flatten, args, []))
         for i, e in enumerate(seq):
             if not isinstance(e, RecoveryPlan):
                 raise ArgumentError("%s (%s) is not of type 'RecoveryPlan'" % (e, type(e)))
@@ -882,7 +894,7 @@ class RecoverySequence(UserList):
                 raise ArgumentError("Recovery step #%d (%s) has unexpected up_index=%d" % (i, e, e.up_index))
             elif e.down_index != len(seq)-1 - i:
                 raise ArgumentError("Recovery step #%d (%s) has unexpected down_index=%d" % (i, e, e.down_index))
-        return super(RecoverySequence, self).__init__(seq)
+        super(RecoverySequence, self).__init__(seq)
 
     def __repr__(self):
         """Use Syntropy serialization."""
