@@ -2,11 +2,12 @@
 #
 # Unclaim disks from a storage pool on an SRN.
 #
+from __future__ import print_function, absolute_import, unicode_literals
 import argparse
 from dgpy.cms_client import CmsClient
 from dgpy.utils      import die
 from dgpy.job        import status_printer
-from dgpy.errors     import LegacyMessageError, GrpcError, NotFoundError, TimeOutError
+from dgpy.errors     import LegacyMessageError, GrpcError, TimeOutError
 
 parser = argparse.ArgumentParser(description="Un-claim @DISK(s) from the LVM storage @POOL at the SRN @IP")
 
@@ -24,7 +25,7 @@ try:
     if args.wait or args.timeout > 0:
         client.wait_for_job(res.job_id, args.timeout, status_printer)
     else:
-        print "Disk release Job ID: {}".format(res.job_id)
+        print("Disk release Job ID: {}".format(res.job_id))
 except (LegacyMessageError, GrpcError) as e:
     die("Failed to release {1} on {0}: {2}".format(args.IP, " and ".join(args.DISK), e))
 except TimeOutError as e:
