@@ -2,6 +2,7 @@
 #
 # Delete a peer node from the peer list of a given SRN.
 #
+from __future__ import print_function, absolute_import, unicode_literals
 import argparse
 from dgpy.cms_client import CmsClient
 from dgpy.utils      import die
@@ -22,7 +23,7 @@ try:
     if args.wait or args.timeout > 0:
         client.wait_for_job(res.job_id, args.timeout, status_printer)
     else:
-        print "Peer de-registration Job ID: {}".format(res.job_id)
+        print("Peer de-registration Job ID: {}".format(res.job_id))
 except NotFoundError as e:
     die("No such node: '{0}'".format(args.DVOL))
 except (LegacyMessageError, GrpcError) as e:
